% HEQKDEMO Solve the H-equation with nsoli.
% This program creates the H-equation example in Chapter 3.
%
function heqkdemo
global A_heq;
nloptions=knl_optset('maxit',40,'maxitl',40,'atol',1.d-8,'rtol',1.d-8,...
'reorth',1,'orthog','cgs');
c=.9;
n=2000;
%
% nodal points for the midpoint rule
%
gr=1:n;
gr=(gr-.5)/n;
gr=gr';
%
% form and store the kernel of the integral operator
%
cc=.5*c/n;
A_heq=ones(n,1)*gr'; A_heq=cc*A_heq'./(A_heq+A_heq');
%
tol=[1.d-8,1.d-8];
legends=cell(3,1);
x=ones(n,1); 
%
% GMRES
%
legends{1}='gmres';
tic
[solk, it_histg, ierrk] = knl(x,@heq,nloptions,A_heq);
toc
%
% BiCGSTAB
%
legends{2}='bicgstab';
nloptions2=knl_optset('x_dependent_jtv',@heqjv,'lmethod','bicgstab');
tic
[sol, it_histb, ierr] = knl(x,@heq,nloptions2,A_heq);
toc
%
% TFQMR
%
legends{3}='tfqmr';
nloptions3=knl_optset('x_dependent_jtv',@heqjv,'lmethod','tfqmr');
tic
[sol, it_histt, ierr] = knl(x,@heq,nloptions3,A_heq);
toc
figure(1)
ng=length(it_histg(:,1));
nk=length(it_histb(:,1));
nt=length(it_histt(:,1));
semilogy(0:ng-1,it_histg(:,1)/it_histg(1,1),'-',...
0:nk-1,it_histb(:,1)/it_histb(1,1),'-.',...
0:nt-1,it_histt(:,1)/it_histt(1,1),'-.');
xlabel('Nonlinear iterations');
ylabel('Relative residual norm');
legend(legends{1},legends{2},legends{3});

function jv = heqjv(v,jac_data)
fx=jac_data.fx; x=jac_data.x;
A_Heq=jac_data.static_data; w=A_Heq*v;
q=x-fx; q=q.*q;
jv=v - q.*w;
